#!/usr/bin/perl
# (C) 2004 Jean-Michel Dault <jmdault@mandrakesoft.com>
# This program is Free Software under the GNU General Public License (>=v2).
# Read the file COPYING that comes with this packages for details.
#
# This script will increment a "autouid" entry to find the next
# available uidNumber for posixAccount
#
# We need to do an atomic operation, meaning that if the old value
# can't be deleted (someone is accessing it at the same time), we 
# don't increment the value, but try again.
#
# It's in Perl because there's no way to do LDAP atomic operations in
# PHP...


#$debug=1;


use URI;
use Net::LDAP;

sub myfatal{
   foreach(@_) { print "$_"; }
   print "\n";
   exit 1;
}

sub mydebug{
   if($debug) {
      print $_[0];
      print "\n";
   }
}

$ldap_uri=$ARGV[0];
$base_dn=$ARGV[1];
$bind_dn=$ARGV[2];
$bind_pw=$ARGV[3];

my $ldapuri = URI->new($ldap_uri) or myfatal "$@";
mydebug "Connecting to ".$ldapuri->host.", port ".$ldapuri->port;
my $ldap = Net::LDAP->new($ldapuri->host, port=> $ldapuri->port) or myfatal "$@";
if ($ldap) {
   mydebug "Connected! Binding as $bind_dn with password $bind_pw";
   $ldap->bind($bind_dn, password=> $bind_pw) || myfatal "$@";
   mydebug "Bound! Now doing a search on autouid";
   my $mesg = $ldap->search(base=> "$base_dn", scope=> 'sub', filter=> "(objectClass=autouid)");
   @entries = $mesg->entries;
   if($mesg->count ==0 ) { 
     myfatal "Error! Cannot find autouid entry!";
   }
   if($mesg->count > 1 ) { 
     myfatal "Error! Multiple autouid entries!";
   }
   @entries=$mesg->entries;
   $origuid=$mesg->entry(0)->get_value('uidNumber');
   $dn=$mesg->entry(0)->dn();
}

if(!$origuid) {
   myfatal "Autouid does not contain uidNumber!";
}

mydebug "Found $origuid";

$newuid=$origuid+1;

do {
   $mesg=$ldap->modify($dn, changes=>[ delete=> [ 'uidNumber'=>$origuid ],
                                          add=> [ 'uidNumber'=>$newuid ] ]);
   if(!$mesg->code) { 
      my $mesg = $ldap->search(base=> "$base_dn", scope=> 'sub', filter=> "(&(uidNumber=$newuid)(!(objectClass=autouid)))");
      @entries = $mesg->entries;
      if($mesg->count ==0 ) { 
        $goodid=$newuid; 
      } else {
         #myfatal "uid already taken $newuid !" ;
      }
      $origuid++;
      $newuid++;
   } else {
      mydebug $mesg->error;
   }
} until $goodid;

print "UIDOK:$goodid\n";
