<?
/*
 (c) 2003 Tassilo Erlewein <tassilo.erlewein@erfrakon.de>
 (c) 2003 Martin Konold <martin.konold@erfrakon.de>
 (c) 2004 Jean-Michel Dault <jmdault@mandrakesoft.com>
 This program is Free Software under the GNU General Public License (>=v2).
 Read the file COPYING that comes with this packages for details.
*/



function email_address_ok ($email,$domain="")
{
   //JMD: From http://karmak.org/archive/2003/02/validemail.html
   $p ="/^(";
   $p.="([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|";
   //The brackets indicate a group of characters. The + at the end says 
   //that the group of characters has to appear at least once, but it can 
   //appear as many times as you want. The group starts out with a carat 
   //(^) character, which indicates negation. So we are listing characters 
   //that CANNOT appear: greater than or less than signs, open or close 
   //parentheses, open or close brackets (the slash before the close bracket 
   //indicates that the character is literal instead of closing the group 
   //of characters), slash (needs to have a slash in front to indicate a 
   //literal character), period, comma, semicolon, colon, a whitespace 
   //character (space, tab, form feed, line feed), at sign, or quote.
   //If any one of those characters appear one or more times, the address 
   //will be not valid. But all other characters not listed are valid 
   //and something outside of that group must appear at least once (because 
   //of the + sign following the group).
   //
   //After one or more valid characters, there is parentheses around another
   //group, followed by a star (*) character. The star idicates that the
   //preceeding group can appear zero or more times. In other words, if it's
   //omitted it's ok, but it can appear. That applies to the whole grouping 
   //of characters. Within this grouping, is a period followed by one or 
   //more valid characters (same listing as before).
   //
   //All this means that there has to be one or more valid characters before 
   //the @ sign, and if there are any periods before the @ sign they must 
   //be followed by one or more valid characters. Since periods themselves are 
   //listed in the not valid characters, a period cannot be the first character 
   //and there cannot be consecutive periods before the @ sign.

   $p.="(\".+\")";

   //This grouping is a bit easier to understand. A literal quote character 
   //can appear, followed by one or more characters (here, the period 
   //indicates any single character) followed by another qoute. So, 
   //"johndoe"@ is a valid starting part of the email address. And if there 
   //truly was a case where consecutive periods were in the part before the
   //@ sign, the whole thing could be enclosed in quotes to make it valid 
   //("john..doe"@ would be a valid first part, but john..doe@ would not be 
   //valid).

   $p.=")@(";

   //After one of the first two groups of characters appears at the start of 
   //the email address, the @ character must appear. Then the next grouping 
   //must appear at the end of the email address (the $ after the big 
   //grouping indicates that). There are other "sub" groups that are "or"ed 
   //together to make up the big grouping.

   //JMD: the rest is what was in kolab originally, except
   //     I concatenate the strings (spaces break pattern).

   $p.="(([[:alnum:]]|[[:alnum:]][[:alnum:]-]*[[:alnum:]])\.)+";

   $p.="(";

   //JMD: deal with the case that someone setup their kolab
   //     server with a fake domain
   if($domain) {
     $suffixes=split('\.',$domain);
     $p.=$suffixes[count($suffixes)-1]."|";
   }

   $p.="com|edu|gov|int|mil|net|org|";

   $p.="aero|arpa|biz|coop|info|museum|name|pro|";

   $p.="ad|ae|af|ag|ai|al|am|an|ao|aq|ar|as|at|au|aw|az|ba|";
   $p.="bb|bd|be|bf|bg|bh|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|";
   $p.="bz|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|co|cr|cs|cu|cv|";
   $p.="cx|cy|cz|de|dj|dk|dm|do|dz|ec|ee|eg|eh|er|es|et|eu|";
   $p.="fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gh|gi|gl|gm|gn|gp|";
   $p.="gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|in|";
   $p.="io|iq|ir|is|it|jm|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|";
   $p.="ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|mg|";
   $p.="mh|mk|ml|mm|mn|mo|mp|mq|mr|ms|mt|mu|mv|mw|mx|my|mz|";
   $p.="na|nc|ne|nf|ng|ni|nl|no|np|nr|nt|nu|nz|om|pa|pe|pf|";
   $p.="pg|ph|pk|pl|pm|pn|pr|ps|pt|pw|py|qa|re|ro|ru|rw|sa|";
   $p.="sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|st|su|sv|";
   $p.="sy|sz|tc|td|tf|tg|th|tj|tk|tm|tn|to|tp|tr|tt|tv|tw|";
   $p.="tz|ua|ug|uk|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|";
   $p.="ye|yt|yu|za|zm|zw)|";

   $p.="(([0-9][0-9]?|[0-1][0-9][0-9]|";
   $p.="[2][0-4][0-9]|[2][5][0-5])\.){3}([0-9][0-9]?|";
   $p.="[0-1][0-9][0-9]|[2][0-4][0-9]|[2][5][0-5])";

   $p.=")$/i";

   return(preg_match($p,$email));

}

function ipaddress_ok ($a)
{
   return preg_match("/(\[0-9]{1,3})\.(\[0-9]{1,3})\.(\[0-9]{1,3})\.(\[0-9]{1,3})/", $a);
}

function domain_ok ($a)
{
   return preg_match("/[a-zA-Z]([a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?\.[a-zA-Z]([a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(\.[a-zA-Z]([a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)?/", $a);
}

//JMD: add '$' to user names to enable interdomain accounts
//JMD: add '.' to user names 
function uid_ok ($a)
{
   return preg_match("/^[a-z0-9\.$]+$/", $a);
}

?>
